(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2017 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  do funkce OpenLayer doplnn parametr Color (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY GCode;

USES SysUtils, DrvTypes, DLL_Util;

{$R VersionInfo.res}

TYPE
  ParamIndex_T = (
    FeedSpeedXY,
    FeedSpeedZ);

CONST
  ToolCount = 0;
  DriverDescr: DriverDescr_T = (
    Name:         'G Code   PH2017';
    MajorVersion: 4;
    MinorVersion: 50;
    Capabilities: [Optimizable, DrawLines, DrawArcs, FlashPads, UserTools, ConfigTools];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'GCode';
    TargetExt:    '.nc';
    NoOfTools:    ToolCount;
    InitialTools: nil;
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           800;
    Y1:           600;
    XDividend:    100;
    YDividend:    100;
    Divisor:      1);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>Feed Speed (Cutting, XY)';
    Value:     100;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    9999),
   (DescrStr:  'Feed Speed (>Plunge, Z)';
    Value:     50;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    9999));

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

// -----------------------------------------------------------------------------

function iSqr (x, y: Integer): Int64;
asm
        push EBX
        mov  ECX, EDX   // y do ECX
        imul EAX
        mov  EBX, EDX   // x^2 do ECX:EBX,
        xchg ECX, EAX   // y do EAX
        imul EAX
        add  EAX, ECX
        adc  EDX, EBX   // EAX:EDX := x^2 + y^2
        pop  EBX
end;

// -----------------------------------------------------------------------------

CONST
  NoTool: ToolDescr_T = (DescrStr: '';
                         Shape:    CircleTool;
                         Width:    MaxInt;
                         Height:   MaxInt;
                         Usage:    [];
                         Code:     0);

VAR
  CurrentX,
  CurrentY,
  CurrentZ,
  CurrentF:         DriverUnits_T;
  CurrentToolCode:  ToolCode_T;
  CurrentTool:      ToolDescr_T;

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
begin
  Write := WriteProc;
  WriteStr ('G90'^M^J);                 // absolutn souadnice
  CurrentX        := MaxInt;
  CurrentY        := MaxInt;
  CurrentZ        := MaxInt;
  CurrentF        := MaxInt;
  CurrentToolCode := MaxInt;
  CurrentTool     := NoTool;
end;

PROCEDURE Comment (const CommentString: ShortString);
begin
end;

PROCEDURE OpenLayer (Layer: integer; Color: integer = 0);
begin
end;

CONST
  ZLevel = 300;

procedure ToolUp;
begin
  WriteStr (Format ('G00 Z%0.2f'^M^J,
                    [ZLevel / 100]));
  CurrentZ := ZLevel;
end;

procedure ToolDown;
  var Speed: integer;
begin
  Speed := Params [FeedSpeedZ].Value;
  if CurrentZ <> 0 then begin
    if Speed > 0 then begin
      WriteStr (Format ('G01 Z%0.2f F%d'^M^J,
                        [0 / 100, Speed]));
      CurrentF := Speed;
    end
    else
      WriteStr (Format ('G01 Z%0.2f'^M^J,
                        [0 / 100]));
    CurrentZ := 0;
  end;
end;

procedure WriteGXYF (G: integer; X, Y: DriverUnits_T; Speed: integer);
begin
  if (Speed > 0) and (Speed <> CurrentF) then begin
    WriteStr (Format ('G%0.2d X%0.2f Y%0.2f F%d'^M^J,
                      [G, X / 100, Y / 100, Speed]));
    CurrentF := Speed;
  end
  else
    WriteStr (Format ('G%0.2d X%0.2f Y%0.2f'^M^J,
                      [G, X / 100, Y / 100]));
  CurrentX := X;
  CurrentY := Y;
end;

procedure WriteGXYIJF (G: integer; X, Y, I, J: DriverUnits_T);
  var Speed: integer;
begin
  ToolDown;
  Speed := Params [FeedSpeedXY].Value;
  if (Speed > 0) and (Speed <> CurrentF) then begin
    WriteStr (Format ('G%0.2d X%0.2f Y%0.2f I%0.2f J%0.2f F%d'^M^J,
                      [G, X / 100, Y / 100, I / 100, J / 100, Speed]));
    CurrentF := Speed;
  end
  else
    WriteStr (Format ('G%0.2d X%0.2f Y%0.2f I%0.2f J%0.2f'^M^J,
                      [G, X / 100, Y / 100, I / 100, J / 100]));
  CurrentX := X;
  CurrentY := Y;
end;

procedure MoveTo (X, Y: DriverUnits_T);
begin
  if (X <> CurrentX) or (Y <> CurrentY) then begin
    ToolUp;
    WriteGXYF (0, X, Y, 0);     // rapid positioning -- rychlost 0 jen pro potlaen vstupu F
  end;
end;

PROCEDURE DrawLine (X0, Y0, X1, Y1: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  MoveTo (X0, Y0);
  ToolDown;
  WriteGXYF (1, X1, Y1, Params [FeedSpeedXY].Value);    // linear interpolation
end;

procedure QuadrantArc (X, Y, R: DriverUnits_T; StartAngle: integer);
  var X0, Y0, X1, Y1: DriverUnits_T;
begin
  case StartAngle of
      0:  begin
            X0 := X + R;
            Y0 := Y;
            X1 := X;
            Y1 := Y + R;
          end;
     90:  begin
            X0 := X;
            Y0 := Y + R;
            X1 := X - R;
            Y1 := Y;
          end;
    180:  begin
            X0 := X - R;
            Y0 := Y;
            X1 := X;
            Y1 := Y - R;
          end;
    270:  begin
            X0 := X;
            Y0 := Y - R;
            X1 := X + R;
            Y1 := Y;
          end;
  end;
  if iSqr (CurrentX - X0, CurrentY - Y0) < iSqr (CurrentX - X1, CurrentY - Y1) then begin
    MoveTo (X0, Y0);
    ToolDown;
    WriteGXYIJF (3, X1, Y1, X - X0, Y - Y0);     // counterclockwise arc
  end
  else begin
    MoveTo (X1, Y1);
    ToolDown;
    WriteGXYIJF (2, X0, Y0, X - X1, Y - Y1);     // clockwise arc
  end;
end {QuadrantArc};

PROCEDURE DrawArc (X, Y, R: DriverUnits_T; Angle1, Angle2: integer; ToolNumber: ToolNumber_T);
begin
  if Angle1 + 90 = Angle2 then
    QuadrantArc (X, Y, R, Angle1)
  else if Angle2 + 90 = Angle1 then
    QuadrantArc (X, Y, R, Angle2)
  else if (Angle1 = 0) and (Angle2 = 360) then begin
    MoveTo (X - R, Y);
    ToolDown;
    WriteGXYIJF (3, X - R, Y, R, 0);             // counterclockwise arc
  end;                                  { TODO : obecn oblouk je ignorovn }
end;

PROCEDURE FlashPad (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
(*
  OutputCoordinates (X, Y);
  WriteStr ('D03*'^M^J);
*)
end;

PROCEDURE DrillHole (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  FlashPad (X, Y, ToolNumber);
end;

PROCEDURE CloseEquip;
begin
  ToolUp;
  WriteStr ('M30'^M^J);                 // absolutn souadnice
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OpenLayer,
  DrawLine,
  DrawArc,
  FlashPad,
  DrillHole,
  CloseEquip;

BEGIN
  FormatSettings.DecimalSeparator := '.';
END.
